// ================================================== //
//     Contact Center Statistics interface module     //
//                     Version 2.2.5                  //
//                                                    //
// ---------- (c) AO SBK 2015-2022 ---------- //
// ================================================== //

var timeframe = {
	firstDate: '',
	secondDate: '',
	firstHour: (getHour() > 0 ? getHour() - 1 : 0),
	secondHour: (getHour() > 22 ? 24 : getHour() + 1)
};
var statistics = {};
var curStatistics = {
  totalAnsw: 0,
  totalUnansw: 0,
  totalMissed: 0,
  totalTransf: 0,
  totalOutAnsw: 0,
  totalOutFailed: 0,
  totalAll: 0,
  data: [],
  graph: [],
  status: {},
};
var inc;
var out;
var ql;
var tw;
var as = [];
var prefilterObjects = [];
var updateTimelineTask;
var playerWindow;

$(document).ready(function() {
  setStatisticsHandlers();
});

function applyFilters() {
  var q = $('#filter-queueName');
  var f = $('#filter-from');
  var t = $('#filter-to');
  var intext = $('#filter-intext').val();
  var queue = (q.val() == q.attr('data-value') || q.val() == '') ? '' : q.val().toLowerCase();
  var from = (f.val() == f.attr('data-value') || f.val() == '') ? '' : f.val().toLowerCase();
  var to = (t.val() == t.attr('data-value') || t.val() == '') ? '' : t.val().toLowerCase();
  if(queue || from || to || intext) {
    var s = {
      totalAnsw: 0,
      totalUnansw: 0,
      totalMissed: 0,
      totalTransf: 0,
      totalOutAnsw: 0,
      totalOutFailed: 0,
      totalAll: 0,
      totalInt: 0,
      totalExt: 0,
      data: [],
      status: {},
    };
    var c = [];
    for(var i = 0; i < curStatistics.data.length; i++) {
      if(queue) {
        if(!isNaN(parseInt(queue))) {
          var qName = getQueueByExt(queue);
          if(qName && curStatistics.data[i].queueName.toLowerCase().indexOf(qName) < 0) { continue; }
        } else if(curStatistics.data[i].queueName.toLowerCase().indexOf(queue) < 0) { continue; }
      }
      if(from && getUserPart(curStatistics.data[i].callFrom).toLowerCase().indexOf(from) < 0) { continue; }
      if(to && getUserPart(curStatistics.data[i].callTo).toLowerCase().indexOf(to) < 0) { continue; }
      if(intext && curStatistics.data[i].intext != intext) { continue; }
      c.push(curStatistics.data[i]);
    }
    // agent statuses
    var filterGroup = [];
    var groups = Object.keys(monitorData.group);
    for(var i = 0; i < groups.length; ++i) {
      if(groups[i].toLowerCase().indexOf(queue) >= 0) {
        filterGroup.push(groups[i]);
      }
    }
    var statuses = Object.keys(statistics.status);
    var iKey;
    for(var i = 0; i < statuses.length; ++i) {
      iKey = statuses[i];
      for(var j = 0; j < filterGroup.length; j++) {
        if(isArrElem(iKey, monitorData.group[filterGroup[j]].agents)) {
          s.status[iKey] = statistics.status[iKey];
          break;
        }
      }
    }
    for(var i = 0; i < statistics.data.length; i++) {
      if(queue) {
        if(!isNaN(parseInt(queue))) {
          var qName = getQueueByExt(queue);
          if(qName && statistics.data[i].queueName.toLowerCase().indexOf(qName) < 0 && statistics.data[i].queueName.toLowerCase().indexOf(queue) < 0) { continue; }
        } else if(statistics.data[i].queueName.toLowerCase().indexOf(queue) < 0) { continue; }
      }
      if(from && getUserPart(statistics.data[i].callFrom).toLowerCase().indexOf(from) < 0) { continue; }
      if(to && getUserPart(statistics.data[i].callTo).toLowerCase().indexOf(to) < 0) { continue; }
      if(intext && statistics.data[i].intext != intext) { continue; }
      if(statistics.data[i].status == 'answered') s.totalAnsw++;
      if(statistics.data[i].status == 'unanswered') s.totalUnansw++;
      if(statistics.data[i].status == 'missed') s.totalMissed++;
      if(statistics.data[i].status == 'transferred') s.totalTransf++;
      if(statistics.data[i].status == 'outanswered') s.totalOutAnsw++;
      if(statistics.data[i].status == 'outfailed') s.totalOutFailed++;
      if(statistics.data[i].intext == 'int') s.totalInt++;
      if(statistics.data[i].intext == 'ext') s.totalExt++;
      s.totalAll++;
      s.data.push(statistics.data[i]);
    }
    if($('#statsGraph').hasClass('statsActive')) {
      createGraphStats(s);
    } else {
      createTableStatsTR(c);
      $('#missedCalls a.yellow').remove();
      updateStatisticsHead(s);
    }
  } else {
    if($('#statsGraph').hasClass('statsActive')) {
      createGraphStats(statistics);
    } else {
      createTableStatsTR(curStatistics.data);
      $('#missedCalls a.yellow').remove();
      updateStatisticsHead(statistics);
    }
  }
  closeOverlay();
}

function setStatisticsHandlers() {
  // statistic
  $(document).on('scroll', function() {
    if(!$('#tabStatistics').hasClass('tabActive')) return;
    if(scrollY > $window.height()) {
      displayUpStats();
    } else {
      $('#upStats').remove();
    }
  });
  $('#prefilter').on('click', function() {
    displayPreFilter();
  });
	$('#tabStatistics .input_box .reset').on('click',function(){
		var t = $(this).siblings('input');
		t.val(t.attr('data-value'));
    t.trigger('keyup');
	});
	$('#tabStatistics .input_box input').on('focus',function(){
		var t = $(this);
		if(t.val()==t.attr('data-value'))
			t.val('');
	}).on('blur',function(){
		var t = $(this);
		if(t.val()=='')
			t.val(t.attr('data-value'));
	});
  $('#tabStatistics .input_box input').on('keyup', function() {
    applyFilters();
  });
  $('#filter-intext').val('');
  select_html($('#filter-intext').parent());
  $('#filter-intext').on('change', function() {
    applyFilters();
  });
  // date
	if ($('#date_from').is('#date_from')) {
		$('#date_from, #date_to').datepicker({
      maxDate: '+0d',
			firstDay: 1,
			dateFormat: 'dd-mm-yy',
			//dayNamesMin: translate('daynames'),
			//monthNames: translate('monthnames'),
		}).on('change',function(){
			$(this).siblings('i').text($(this).val());
      if(this.id == 'date_from') timeframe.firstDate = $(this).val();
      else timeframe.secondDate = $(this).val();
      //if(timeframe.firstDate == timeframe.secondDate) $('#slider_time24').css('visibility', 'visible');
      //else $('#slider_time24').css('visibility', 'hidden');
		});
		$('.date i').on('click',function(){
			$(this).siblings('input').trigger('focus');
		});
    $('#date_from, #date_to').datepicker('setDate', getToday());
    $('#date_from, #date_to').trigger('change');
	}

  // slider
	if ($('#slider_time24').is('#slider_time24')) {
		$('#slider_time24').slider({
			range: true,
			min: 0,
			max: 48,
			values: [timeframe.firstHour*2, timeframe.secondHour*2],
			slide: function(event,ui) {
				if(ui.values[0] == ui.values[1])
					return false;
				var handle = $('#slider_time24 .ui-slider-handle');
				handle.first().text((ui.values[0]/2).toString().replace(/\.5/, ':30'));
				handle.last().text((ui.values[1]/2).toString().replace(/\.5/, ':30'));
        timeframe.firstHour = ui.values[0]/2;
        timeframe.secondHour = ui.values[1]/2;
        if($('#mainTimelineContainer').is(':visible')) {
          clearTimeout(updateTimelineTask);
          updateTimelineTask = setTimeout(function() {
            $('#timelineGraphsTabContainer').html('<h2>' + timeframe.firstDate + '</h2>' +
              '<div id="mainTimelineContainer"><div class="tableRow"><div class="tableCell labelCell"></div>' +
              '<div class="tableCell dataCell"><div id="timelineScaleContainer" class="timelineTimespanContainer">' +
              '</div></div></div></div>');
            processAgentTimeline(statistics.agentStatusTimeline);
          },250);
        }
			}
		});
		sep(48,$('#slider_time24'));
    $('#slider_time24 .ui-slider-handle').first().text(timeframe.firstHour);
    $('#slider_time24 .ui-slider-handle').last().text(timeframe.secondHour);
	}

  $('#str_req').on('click', function(){
    $('#prefilterSettings .closeIcon').trigger('click');
    getStatistics();
  });
  $('#str_expex').on('click', function() {
    $('#prefilterSettings .closeIcon').trigger('click');
    //getStatistics('disp');
    getStatisticsReport(getReportParams());
  });
  $('#str_expst').on('click', function() {
    $('#prefilterSettings .closeIcon').trigger('click');
    getStatistics('expst');
  });
  $('#str_expgr').on('click', function() {
    $('#prefilterSettings .closeIcon').trigger('click');
    if($('#callsGraphsTab').hasClass('bg2')) {
      inc = $('#placeholder0').jqplotToImageStr({backgroundColor: '#222C36'});
      out = $('#placeholder2').jqplotToImageStr({backgroundColor: '#222C36'});
      ql = $('#placeholder1').jqplotToImageStr({backgroundColor: '#222C36'});
      tw = $('#placeholder3').jqplotToImageStr({backgroundColor: '#222C36'});
    } else if($('#statusGraphsTab').hasClass('bg2')) {
      as = [];
      if(statistics.status) {
        var statuses = Object.keys(statistics.status);
        if(statuses.length) {
          for(var i = 0; i < statuses.length; ++i) {
            as.push($('#stgr_' + escapeID(statuses[i])).jqplotToImageStr({backgroundColor: '#222C36'}));
          }
        }
      }
    } else if($('#timelineGraphsTab').hasClass('bg2')) {
      return;
    }
    var f = document.forms['expGraph'];
    f.submit();
  });
  $('#stats').on('click',function() {
    $('#prefilterSettings .closeIcon').trigger('click');
    if(!sessionParams.isDomainAdmin && !sessionParams.access.monitor.length) {
      return;
    }
    createOverlay();
    $('#statsTable').toggleClass('statsActive');
    $('#statsGraph').toggleClass('statsActive');
    if($(this).text() == translate('graphs')) {
      $(this).text(translate('statistics'));
      //$('#filter-to').siblings('.reset').trigger('click');
      //$('#filter-to').parent().hide();
      $('#str_expex').hide();
      $('#str_expgr').show();
    }
    else if($(this).text() == translate('statistics')) {
      $(this).text(translate('graphs'));
      $('#str_expex').show();
      $('#str_expgr').hide();
      //$('#filter-to').parent().show();
      //getStatistics();
    }
    $('#tabStatistics .input_box input').trigger('keyup');
  });
  // tab
	$('#statsTable .table_tab li').on('click',function() {
    createOverlay();
		var t = $(this);
		t.addClass('bg2').siblings('.bg2').removeClass('bg2');
    var filter;
    switch (this.id) {
      case 'allCalls':
        createTableStatsTR(statistics.data);
        curStatistics = statistics;
        $('#tabStatistics .input_box input').trigger('keyup');
        return;
      case 'answCalls':
        filter = 'answered';
        break;
      case 'unanswCalls':
        filter = 'unanswered';
        break;
      case 'missedCalls':
        filter = 'missed';
        break;
      case 'transfCalls':
        filter = 'transferred';
        break;
      case 'outCalls':
        filter = 'outanswered';
        break;
    }
    curStatistics = {
      totalAnsw: 0,
      totalUnansw: 0,
      totalMissed: 0,
      totalTransf: 0,
      totalOutAnsw: 0,
      totalOutFailed: 0,
      totalAll: 0,
      data: [],
    };
    for(var i = 0; i < statistics.data.length; i++) {
      if(filter == 'outanswered') {
        if(statistics.data[i].status == 'outanswered' || statistics.data[i].status == 'outfailed') {
          curStatistics.data.push(statistics.data[i]);
        }
      } else {
        if(statistics.data[i].status == filter) {
          curStatistics.data.push(statistics.data[i]);
        }
      }
    }
    createTableStatsTR(curStatistics.data);
    closeOverlay();
    if(curStatistics.data.length > 0) {
      $('#tabStatistics .input_box input').trigger('keyup');
    }
	});
  $('#statsGraph .table_tab li').on('click', function() {
		var t = $(this);
		t.addClass('bg2').siblings('.bg2').removeClass('bg2');
    $('#' + this.id + 'Container').show().siblings('div').hide();
    applyFilters();
  });
}

function getToday () {
	var date = new Date();
	var day = date.getDate();
	if (day < 10) day = '0' + day;
	var month = date.getMonth() + 1;
	if (month < 10) month = '0' + month;
	var yr = date.getFullYear();
	today = day + '-' + month + '-' + yr;
	var firstHour = timeframe.firstHour;
	var secondHour = timeframe.secondHour;
	return today;
}

function getHour() {
  var date = new Date();
  return date.getHours();
}

function getReportParams() {
  var firstDate = timeframe.firstDate;
	var secondDate = timeframe.secondDate;
	var firstHour = timeframe.firstHour.toString().replace(/\.5/, ':30');
	var secondHour = timeframe.secondHour.toString().replace(/\.5/, ':30');
  var today = getToday();
  var frameFrom = 0;
  var frameTo = 0;
	if (firstDate == '') { firstDate = today; timeframe.firstDate = today; }
	if (secondDate == '') { secondDate = today; timeframe.secondDate = today; }
	if (firstDate != secondDate) {
		firstHour = '00:00:00';
		secondHour = '23:59:59';
    frameFrom = timeframe.firstHour * 3600;
    frameTo = timeframe.secondHour * 3600;
	}
	if(/:/.test(firstHour)) {
    if(firstHour.length < 5) firstHour = '0' + firstHour;
  } else {
    if(firstHour.length < 2) firstHour = '0' + firstHour;
    firstHour += ':00';
  }
  if(firstHour.length < 8) firstHour += ':00';
  if(/:/.test(secondHour)) {
    if(secondHour.length < 5) secondHour = '0' + secondHour;
  } else {
    if(secondHour.length < 2) secondHour = '0' + secondHour;
    secondHour += ':00';
  }
  if(secondHour.length < 8) secondHour += ':00';
	var startDate = '#T' + firstDate + '_' + firstHour;
  var endDate;
	if(secondHour == '24:00:00') endDate = '#T' + secondDate + '_23:59:59';
	else endDate = '#T' + secondDate + '_' + secondHour;
  var objArr;
  if(prefilterObjects.length) {
    objArr = prefilterObjects;
  } else {
    objArr = sessionParams.isDomainAdmin ? ['anyOne'] : getFullObjArr();
  }
  var params = {};
  params.objects = objArr;
  params.accountName = sessionParams.name;
  params.from = startDate;
  params.to = endDate;
  params.frameFrom = frameFrom;
  params.frameTo = frameTo;
  return params;
}

function getStatistics(disp, cookie) {
  createOverlay();
  $('#moreStats').parent().remove();
	var firstDate = timeframe.firstDate;
	var secondDate = timeframe.secondDate;
	var firstHour = timeframe.firstHour.toString().replace(/\.5/, ':30');
	var secondHour = timeframe.secondHour.toString().replace(/\.5/, ':30');
  var today = getToday();
  var frameFrom = 0;
  var frameTo = 0;
	if (firstDate == '') { firstDate = today; timeframe.firstDate = today; }
	if (secondDate == '') { secondDate = today; timeframe.secondDate = today; }
	if (firstDate != secondDate) {
		firstHour = '00:00:00';
		secondHour = '23:59:59';
    frameFrom = timeframe.firstHour * 3600;
    frameTo = timeframe.secondHour * 3600;
	}
	if(/:/.test(firstHour)) {
    if(firstHour.length < 5) firstHour = '0' + firstHour;
  } else {
    if(firstHour.length < 2) firstHour = '0' + firstHour;
    firstHour += ':00';
  }
  if(firstHour.length < 8) firstHour += ':00';
  if(/:/.test(secondHour)) {
    if(secondHour.length < 5) secondHour = '0' + secondHour;
  } else {
    if(secondHour.length < 2) secondHour = '0' + secondHour;
    secondHour += ':00';
  }
  if(secondHour.length < 8) secondHour += ':00';
	var startDate = '#T' + firstDate + '_' + firstHour;
  var endDate;
	if(secondHour == '24:00:00') endDate = '#T' + secondDate + '_23:59:59';
	else endDate = '#T' + secondDate + '_' + secondHour;
  var objArr;
  if(prefilterObjects.length) {
    objArr = prefilterObjects;
  } else {
    objArr = sessionParams.isDomainAdmin ? [] : getFullObjArr();
  }
  if(disp) {
    var op = 'getStatistics';
    if(disp == 'expst') {
      op = 'getStatusStats';
      for(var i = 0; i < objArr.length; ++i) {
        var name = objArr[i];
        if(monitorData.group[name]) {
          objArr.splice(i--, 1);
          var agents = Object.keys(monitorData.group[name].agents);
          var jKey;
          for(var j = 0; j < agents.length; ++j) {
            jKey = agents[j];
            if(objArr.indexOf(monitorData.group[name].agents[jKey]) == -1) {
              objArr.push(monitorData.group[name].agents[jKey]);
            }
          }
        }
      }
    }
    console.log('getStatusStats: from=' + startDate + ' to=' + endDate + ' frameFrom=' + frameFrom + ' frameTo=' + frameTo + ' objects=' + JSON.stringify(objArr));
    window.open(
      ccstatistics +
      '?session=' + encodeURIComponent(sessionId) +
      '&from=' + encodeURIComponent(startDate) +
      '&to=' + encodeURIComponent(endDate) +
      (frameFrom || frameTo ? '&frameFrom=' + encodeURIComponent(frameFrom) + '&frameTo=' + encodeURIComponent(frameTo) : '') +
      '&op=' + op +
      '&objects=' + encodeURIComponent(JSON.stringify(objArr)) +
      '&disp=1',
      '_blank'
    );
    closeOverlay();
  } else {
    var params = {
      op: 'getStatistics',
      session: sessionId,
      from: startDate,
      to: endDate,
      frameFrom: frameFrom,
      frameTo: frameTo,
      objects: JSON.stringify(objArr),
    };
    if(cookie) {
      params.cookie = cookie;
    }
    $.post(
      ccstatistics,
      params,
      function(data) {
        var result = $.parseJSON(data);
        if(result.cutted) {
          $('#tabStatistics .table_stat').after('<p class="center"><button class="ff0 button" id="moreStats">' + translate('more') + '</button></p>');
          $('#moreStats').on('click', function() {
            var cookie = JSON.stringify(result.data[result.data.length - 1]);
            getStatistics(null, cookie);
            $(this).parent().remove();
          });
        }
        if(result.part) {
          var resultKeys = Object.keys(result);
          for(var i = 0; i < resultKeys.length; ++i) {
            if(resultKeys[i] == 'data') {
              statistics.data = statistics.data.concat(result.data);
            } else {
              statistics[resultKeys[i]] = result[resultKeys[i]];
            }
          }
        } else {
          statistics = result;
        }
        curStatistics = statistics;
        $('#missedCalls a.yellow').remove();
        updateStatisticsHead(result);
        createTableStatsTR(result.data, result.part);
        if($('#tabStatistics .table_tab .bg2').attr('id') != 'allCalls') {
          $('#tabStatistics .table_tab .bg2').trigger('click');
        }
        $('#tabStatistics .input_box input').trigger('keyup');
        createGraphStats(result);
        closeOverlay();
      }
    ).fail(function(jqXHR) {
      alert(translate(['statsErrors', jqXHR.responseText]));
      console.log(jqXHR.status + ', ' + jqXHR.statusText + ', ' + jqXHR.responseText);
      closeOverlay();
    });
  }
}

function createTableStatsTR(statsArr, append) {
	var tmpStr = '';
	for (var i = 0; i < statsArr.length; i++) {
    if(statsArr[i] != statsArr[i-1]) {
      var className = escapeID(statsArr[i].callID);
      if (statsArr[i].status == 'missed') className += ' c69';
      tmpStr += '<tr class="' + className + '">';
      tmpStr += '<td nowrap="nowrap">' + statsArr[i].date.substring(0,5) + '</td>';
      tmpStr += '<td nowrap="nowrap">' + statsArr[i].date.substring(11,19) + '</td>';
      if(statsArr[i].queueName.length > 20) {
        tmpStr += '<td><span title="' + statsArr[i].queueName + '">' + statsArr[i].queueName.substring(0,20) + '...</span></td>';
      } else {
        tmpStr += '<td>' + statsArr[i].queueName + '</td>';
      }
      tmpStr += '<td>' + statsArr[i].queueLength + '</td>';
      if(getUserPart(statsArr[i].callFrom).length > 12) {
        tmpStr += '<td><span data-value="' + statsArr[i].callFrom + '" class="cb_num" title="' + getUserPart(statsArr[i].callFrom) + '">' + getUserPart(statsArr[i].callFrom).substring(0,12) + '...</span></td>';
      } else {
        tmpStr += '<td data-value="' + statsArr[i].callFrom + '" class="cb_num">' + getUserPart(statsArr[i].callFrom) + '</td>';
      }
      if(getUserPart(statsArr[i].callTo).length > 12) {
        tmpStr += '<td><span data-value="' + statsArr[i].callTo + '" class="cb_num" title="' + getUserPart(statsArr[i].callTo) + '">' + getUserPart(statsArr[i].callTo).substring(0,12) + '...</span></td>';
      } else {
        tmpStr += '<td data-value="' + statsArr[i].callTo + '" class="cb_num">' + getUserPart(statsArr[i].callTo) + '</td>';
      }
      tmpStr += '<td>' + statsArr[i].tWait + '</td>';
      tmpStr += '<td>' + statsArr[i].tAlert + '</td>';
      tmpStr += '<td>' + statsArr[i].tConnect + '</td>';
      tmpStr += '<td>';
      if (statsArr[i].status == 'answered') tmpStr += translate('Answered');
      else if (statsArr[i].status == 'missed') tmpStr += translate('Missed');
      else if (statsArr[i].status == 'unanswered') tmpStr += translate('Unanswered');
      else if (statsArr[i].status == 'transferred') tmpStr += translate('Transferred');
      else if (statsArr[i].status == 'outanswered') tmpStr += translate('outgoing');
      else if (statsArr[i].status == 'outfailed') tmpStr += translate('failed');
      else tmpStr += statsArr[i].status;
      tmpStr += statsArr[i].statusInfo ? " " + statsArr[i].statusInfo : "";
      tmpStr += '</td>';
      var urlFile = '&nbsp;';
      if (typeof statsArr[i].fileURL != 'undefined' && statsArr[i].fileURL != '-') {
        urlFile = '/Session/' + sessionId + '/DOWNLOAD/' + statsArr[i].fileURL;
        urlFile = encodeURI(urlFile).replace(/#/g, '%23');
        tmpStr += '<td><a class="rec" href="' + urlFile + '" target="_blank"></a></td>';
      } else if(statsArr[i].status == 'answered' || statsArr[i].status == 'outanswered') {
        tmpStr += '<td><a class="rec off"></a></td>';
      } else tmpStr += '<td></td>';
      tmpStr += '<td>' + (statsArr[i].comment == '-' || !/\//.test(statsArr[i].comment) ? '' : '<a class="info ivrComment pointer" data-value="' + statsArr[i].comment.replace(/"/g, '&quot;') + '"></a>') + (statsArr[i].notes && statsArr[i].notes != '-' ? ' <a class="info callNotes pointer" data-value="' + statsArr[i].notes.replace(/"/g, '&quot;') + '"></a>' : '') + '</td>';
		  tmpStr += '</tr>';
		}
	}
  if(append) {
  	$('#statsTableTR').append(tmpStr);
    $('#statsTableTR a').off('click');
    $('#statsTableTR tr').off('click');
    $('#statsTableTR .cb_num').off('click');
  } else {
  	$('#statsTableTR').html(tmpStr);
  }
  $('#statsTableTR a.rec').on('click', function(e) {
    if($(this).hasClass('off')) return;
    e.preventDefault();
    audioPlayerWindow(this.href);
  });
  $('#statsTableTR a.info.ivrComment').on('click', function() {
    var comment = $(this).attr('data-value');
    var cArr = comment.split('/');
    var ivrLog = new Array();
    for(var i = 0; i < cArr.length; i++) {
      var ccArr = cArr[i].split(';');
      var logRecord = {
        time: ccArr[0],
        action: ccArr[1],
        param: ccArr[2],
      };
      ivrLog.push(logRecord);
    }
    var date = $(this).parent().siblings('td').eq(0).text();
    var from = $(this).parent().siblings('td').eq(4).text();
    displayIvrLog(date, from, ivrLog);
  });
  $('#statsTableTR a.info.callNotes').on('click', function() {
    var notes = JSON.parse($(this).attr('data-value'));
    var date = $(this).parent().siblings('td').eq(0).text();
    var from = $(this).parent().siblings('td').eq(4).text();
    displayCallNotes(date, from, notes);
  });
  $('#statsTableTR tr').on('click',function(){
    var classSel = '.' + $(this).attr('class').split(/\s+/)[0];
    $(classSel).siblings('.sel:not(' + classSel + ')').removeClass('sel');
    $(classSel).toggleClass('sel');
  });
  $('#statsTableTR .cb_num').on('click', function() {
    var num = $(this).attr('data-value');
    if(/,/.test(num) || num == sessionParams.name) {
      $('#callbackButton').hide();
      return;
    }
    num = num.replace(/[()]/g, '');
    $('#cb_num').attr('data-value', num);
    num = num.replace(/@.*$/, '');
    $('#cb_num').text(num);
    $('#callbackButton').show();
  });
}

function audioPlayerWindow(fileRef) {
  if(playerWindow) playerWindow.close();
  var filePathArr = fileRef.split('/');
  var fileName = fileRef;
  if(filePathArr.length > 0) fileName = filePathArr[filePathArr.length-1];
  playerWindow = window.open('about:blank', 'record', 'top=100,left=100,width=800,height=150,menubar=no,toolbar=no,location=no,status=no');
  if(playerWindow === null) return;
  playerWindow.document.write('<html><head><title>' + translate('record') + '</title></head>' +
    '<body style="background:#222C36;color:#FFF;"><h2>' + fileName + '</h2><p><audio controls src="' + fileRef + '"></audio></p>' +
    '<p><a style="color:#FFF;" href="' + fileRef + '">' + translate('download') + '</a></p></body></html>');
}

function escapeID(id) {
  return encodeURIComponent(id).replace(
    /([\#\@|\;|\&|\,|\.|\+|\*|\~|\'|\:|\"|\!|\^|\$|\[|\]|\(|\)|\=|\>|\||\/|\%])/g, ''
  );
}

function createGraphStats(statsObj) {
	$('#placeholder0, #placeholder1, #placeholder2, #placeholder3, #statusGraphContainer').html('');
  var currentTab = $('#grafic .table_tab .bg2');
  inc = null;
  out = null;
  ql = null;
  tw = null;
  as = [];
  if(currentTab.attr('id') == 'callsGraphsTab') {
    var timeInc = 0;
    var timeOut = 0;
    for(var i = 0; i < statsObj.data.length; i++) {
      if(!/\-/.test(statsObj.data[i].tConnect)) {
        var m = statsObj.data[i].tConnect.replace(/^(\d\d*)\:(\d\d)$/, '$1');
        var s = statsObj.data[i].tConnect.replace(/^(\d\d*)\:(\d\d)$/, '$2');
        var t = parseInt(m)*60 + parseInt(s);
        if(statsObj.data[i].status == 'answered') {
          timeInc += t;
        }
        if(statsObj.data[i].status == 'outanswered') {
          timeOut += t;
        }
      }
    }
    $('#str_graphinc').siblings('span').text(translate('talktime') + ' ' + secToTimeStr(timeInc));
    $('#str_graphout').siblings('span').text(translate('talktime') + ' ' + secToTimeStr(timeOut));
    if(
      !isNaN(statsObj.totalAnsw) &&
      !isNaN(statsObj.totalUnansw) &&
      !isNaN(statsObj.totalMissed) &&
      !isNaN(statsObj.totalAll)
    ) {
      var s1 = [[translate('Answered'), parseInt(statsObj.totalAnsw) + parseInt(statsObj.totalTransf)],[translate('Unanswered'), parseInt(statsObj.totalUnansw)],[translate('Missed'), parseInt(statsObj.totalMissed)]];
      var maxTotalAll = parseInt(statsObj.totalAll);
      maxTotalAll = Math.ceil(maxTotalAll * 1.1); // +10%
      while( (maxTotalAll % 3) != 0 ) { maxTotalAll++; }
      if (!(s1[0,1] == 0 && s1[1,1] == 0 && s1[2,1] == 0)) {
        plot0 = $.jqplot('placeholder0', [s1], {
          //title: { text: translate('incoming'), fontSize: '14px' },
          grid: { background: 'transparent', gridLineColor: '#999999', gridLineWidth: 0.2 },
          seriesColors: ['#66FFFF', '#CC6699', '#8967AC'],
          seriesDefaults: {
            renderer: $.jqplot.BarRenderer,
            rendererOptions: { varyBarColor: true	},
            pointLabels: { show: true }
          },
          axes: {
            xaxis: { renderer: $.jqplot.CategoryAxisRenderer },
            yaxis: { numberTicks: 4, label: translate('calls'), },
          }
        });
        //inc = $('#placeholder0').jqplotToImageStr({backgroundColor: '#222C36'});
      }
    }
    if(!isNaN(statsObj.totalOutAnsw) && !isNaN(statsObj.totalOutFailed)) {
      var omaxTotalAll = parseInt(statsObj.totalOutAnsw) > parseInt(statsObj.totalOutFailed) ? parseInt(statsObj.totalOutAnsw) : parseInt(statsObj.totalOutFailed);
      omaxTotalAll = Math.ceil(omaxTotalAll * 1.1); // +10%
      while( (omaxTotalAll % 3) != 0 ) { omaxTotalAll++; }
      var os1 = [[translate('connected'), parseInt(statsObj.totalOutAnsw)],[translate('failed'), parseInt(statsObj.totalOutFailed)]];
      if (!(os1[0,1] == 0 && os1[1,1] == 0)) {
        plot2 = $.jqplot('placeholder2', [os1], {
          //title: { text: translate('outgoing'), fontSize: '14px' },
          grid: { background: 'transparent', gridLineColor: '#999999', gridLineWidth: 0.2 },
          seriesColors: ['#66FFFF', '#CC6699'],
          seriesDefaults: {
            renderer: $.jqplot.BarRenderer,
            rendererOptions: { varyBarColor: true	},
            pointLabels: { show: true }
          },
          axes: {
            xaxis: { renderer: $.jqplot.CategoryAxisRenderer },
            yaxis: { numberTicks: 4, label: translate('calls'), },
          }
        });
        //out = $('#placeholder2').jqplotToImageStr({backgroundColor: '#222C36'});
      }
    }
    
    if(statsObj.graph && statsObj.graph.length) {
      var line1 = [];
      var valArr = [];
      var line2 = [];
      var qwt = [];
      var maxwt = 0;
      var maxQueue = 0;
      for (var i = 0; i < statsObj.graph.length; i++) {
        var tmpArr = [];
        if(statsObj.graph[i].tEnqueue != '-') {
          tmpArr[0] = statsObj.graph[i].tEnqueue.substring(3,5) + '-' + statsObj.graph[i].tEnqueue.substring(0,2) + '-' + statsObj.graph[i].tEnqueue.substring(6,10) + ' ' + statsObj.graph[i].tEnqueue.substring(11,19);
          tmpArr[1] = +statsObj.graph[i].queueLength - 1;
          if(!isNaN(tmpArr[1])) {
            line1.push(tmpArr);
            valArr.push(tmpArr[1]);
          }
        }
        if(statsObj.graph[i].tDequeue != '-') {
          tmpArr[0] = statsObj.graph[i].tDequeue.substring(3,5) + '-' + statsObj.graph[i].tDequeue.substring(0,2) + '-' + statsObj.graph[i].tDequeue.substring(6,10) + ' ' + statsObj.graph[i].tDequeue.substring(11,19);
          tmpArr[1] = +statsObj.graph[i].qLengthDequeue;
          if(!isNaN(tmpArr[1])) {
            line1.push(tmpArr);
            valArr.push(tmpArr[1]);
          }
        }
        var tWait = statsObj.graph[i].tWait;
        if(tWait && statsObj.graph[i].status != 'unanswered') {
          var secwait = +tWait.substring(0, tWait.indexOf(':')) * 60 +
            +tWait.substring(tWait.indexOf(':') + 1);
          if(!isNaN(secwait)) {
            var date = statsObj.graph[i].date.substring(3,5) + '-' + statsObj.graph[i].date.substring(0,2) + '-' + statsObj.graph[i].date.substring(6,10) + ' ' + statsObj.graph[i].date.substring(11,19);
            line2.push([date,secwait]);
            qwt.push(secwait);
          }
        }
      }
      if (!line1.length) {
        var date = timeframe.firstDate;
        var hour = timeframe.firstHour > 10 ? timeframe.firstHour.toString() + ':00:00' : '0' + timeframe.firstHour.toString() + ':00:00';

        date = date.substring(3,5) + '-' + date.substring(0,2) + '-' + date.substring(6,10) + ' ' + hour;
        line1.push([date,0]);
        date = timeframe.secondDate;
        hour = timeframe.secondHour > 10 ? (timeframe.secondHour == 24 ? '23:59:59' : timeframe.secondHour.toString() + ':00:00') : '0' + timeframe.secondHour.toString() + ':00:00';
        date = date.substring(3,5) + '-' + date.substring(0,2) + '-' + date.substring(6,10) + ' ' + hour;
        line1.push([date,0]);
        valArr = [0,0];
      }
      if(!line2.length) {
        var date = timeframe.firstDate;
        var hour = timeframe.firstHour > 10 ? timeframe.firstHour.toString() + ':00:00' : '0' + timeframe.firstHour.toString() + ':00:00';
        date = date.substring(3,5) + '-' + date.substring(0,2) + '-' + date.substring(6,10) + ' ' + hour + '00:00';
        line2.push([date,0]);
        date = timeframe.secondDate;
        hour = timeframe.secondHour > 10 ? (timeframe.secondHour == 24 ? '23:59:59' : timeframe.secondHour.toString() + ':00:00') : '0' + timeframe.secondHour.toString() + ':00:00';
        date = date.substring(3,5) + '-' + date.substring(0,2) + '-' + date.substring(6,10) + ' ' + hour + '00:00';
        line2.push([date,0]);
        qwt = [0,0];
      }

      maxQueue = Math.max.apply({},valArr);
      maxQueue++;
      maxwt = Math.max.apply({},qwt);
      maxwt++;
      
      while( (maxQueue % 4) != 0 && !isNaN(maxQueue % 4) ) { maxQueue++; }
      while( (maxwt % 4) != 0 && !isNaN(maxwt % 4) ) { maxwt++; }

      var format = timeframe.firstDate != timeframe.secondDate ? '%d-%m-%Y %H:%M' : '%H:%M';
      $('#placeholder1').css('height', 300);
      var plot1 = $.jqplot('placeholder1', [line1], {
        //title: { text: translate('queuelength'), fontSize: '14px' },
        grid: { background: 'transparent', gridLineColor: '#999999', gridLineWidth: 0.2 },
        seriesColors:['#8967AC'],
        axes:{
        xaxis:{
          renderer:$.jqplot.DateAxisRenderer,
          tickOptions:{formatString:format},
        },
        yaxis: {
          min:0, max:maxQueue, label: translate('calls'),
        }
        },
        highlighter: {
        show: true,
        sizeAdjust: 7.5
        },
        cursor: {
        show: false
        }
      });
      $('#placeholder1').css('height', 330);
      //ql = $('#placeholder1').jqplotToImageStr({backgroundColor: '#222C36'});
      if(line2.length) {
        $('#placeholder3').css('height', 300);
        var plot3 = $.jqplot('placeholder3', [line2], {
          //title: { text: translate('timewait'), fontSize: '14px' },
          grid: { background: 'transparent', gridLineColor: '#999999', gridLineWidth: 0.2 },
          seriesColors:['#FF0'],
          seriesDefaults:{
            renderer:$.jqplot.BarRenderer,
            rendererOptions: {
              barWidth: 2,
              shadow: false
            },
          },
          axes:{
          xaxis:{
            renderer:$.jqplot.DateAxisRenderer,
            tickOptions:{formatString:format},
          },
          yaxis: {
            min:0, max:maxwt, label: translate('seconds'),
          }
          },
          highlighter: {
          show: true,
          sizeAdjust: 7.5
          },
          cursor: {
          show: false
          }
        });
        $('#placeholder3').css('height', 330);
        //tw = $('#placeholder3').jqplotToImageStr({backgroundColor: '#222C36'});
      }
    }
    // fix yaxis label position
    $('.jqplot-yaxis-label').css('left', '-15px');
  } else if(currentTab.attr('id') == 'statusGraphsTab') {
    if(statsObj.status && getKeysCount(statsObj.status)) {
      var statusKeys = Object.keys(statsObj.status);
      var iKey;
      for(var i = 0; i < statusKeys.length; ++i) {
        iKey = statusKeys[i];
        // time on phone
        var onPhone = 0;
        for(var j = 0; j < statsObj.data.length; j++) {
          if((statsObj.data[j].callFrom == iKey || statsObj.data[j].callTo == iKey) && statsObj.data[j].tConnect != '-') {
            var m = statsObj.data[j].tConnect.replace(/^(\d\d*)\:(\d\d)$/, '$1');
            var s = statsObj.data[j].tConnect.replace(/^(\d\d*)\:(\d\d)$/, '$2');
            var t = parseInt(m)*60 + parseInt(s);
            onPhone += t;
          }
        }
        // onPhone
        var onPhoneStr = secToTimeStr(onPhone);
        // on
        var onStr = secToTimeStr(statsObj.status[iKey].on);
        // off
        var offStr = secToTimeStr(statsObj.status[iKey].off);
        // break
        var breakStr = secToTimeStr(statsObj.status[iKey].break);
        // acw
        var acwStr = secToTimeStr(statsObj.status[iKey].acw);
        $('#statusGraphContainer').append('<div id="stgr_' + escapeID(iKey) + '" class="statusGraph"></div>');
        var data = [
          [translate(['agentStatus', 'on']) + '<br/>' + onStr, statsObj.status[iKey].on],
          [translate(['agentStatus', 'off']) + '<br/>' + offStr, statsObj.status[iKey].off],
          [translate(['agentStatus', 'break']) + '<br/>' + breakStr, statsObj.status[iKey].break],
          [translate(['agentStatus', 'acw']) + '<br/>' + acwStr, statsObj.status[iKey].acw]
        ];
        drawEgg('stgr_' + escapeID(iKey),data,iKey,onPhoneStr);
        //as.push($('#stgr_' + escapeID(i)).jqplotToImageStr({backgroundColor: '#222C36'}));
      }
    }
  } else if(currentTab.attr('id') == 'timelineGraphsTab') {
    $('#timelineGraphsTabContainer').html('<div class="loader"></div>');
    var objArr = prefilterObjects.length ? prefilterObjects : (sessionParams.isDomainAdmin ? [] : getFullObjArr());
    $.get(ccstatistics, {op: "getAgentStatusTimeline", session: sessionId, date: '#T'+timeframe.firstDate, objects: JSON.stringify(objArr) }, function(data) {
      $('#timelineGraphsTabContainer').html('<h2>' + timeframe.firstDate + '</h2><div id="mainTimelineContainer"><div class="tableRow"><div class="tableCell labelCell"></div>' +
        '<div class="tableCell dataCell"><div id="timelineScaleContainer" class="timelineTimespanContainer"></div></div></div></div>');
      var result = JSON.parse(data);
      processAgentTimeline(result);
    }).fail(function(jqXHR) {
      $('#timelineGraphsTabContainer').html('');
      //alert(translate(['statsErrors', jqXHR.responseText]));
      console.log(jqXHR.status + ', ' + jqXHR.statusText + ', ' + jqXHR.responseText);
    });
  }
}

function drawEgg(containerId,data,title,talkTime) {
  setTimeout(function() {
    if(document.getElementById(containerId) == null) return;
    $.jqplot(containerId, [data], {
      title: { text: title + '<span class="right">' + translate('talktime') + ' ' + talkTime + '</span>', textAlign: 'left', fontSize: '14px' },
      grid: { background: 'transparent', gridLineColor: '#999999', gridLineWidth: 0.2 },
      seriesDefaults: {
        renderer: $.jqplot.PieRenderer, 
        rendererOptions: {
          showDataLabels: true,
        }
      },
      seriesColors: ['#66FFFF', '#CC6699', '#8967AC', '#c7ca0d'],
      legend: { show: true, location: 'e' },
    });
  },0);
}

function updateStatisticsHead(statsObj) {
  if(!getKeysCount(statsObj)) {
    $('#answCalls a').text('');
    $('#unanswCalls a').text('');
    $('#missedCalls a').text('');
    $('#allCalls a').text('');
    $('#transfCalls a').text('');
    $('#outAnswCalls a').text('');
    $('#outFailedCalls a').text('');
  } else {
    $('#answCalls a').text(statsObj.totalAnsw);
    $('#unanswCalls a').text(statsObj.totalUnansw);
    //$('#missedCalls a.yellow').remove();
    var percent = Math.round(100*statsObj.totalMissed/(statsObj.totalAnsw + statsObj.totalMissed));
    $('#missedCalls a').text(statsObj.totalMissed).after('<a class="yellow" style="font-size: 14px; margin-left: 10px;">' + (isNaN(percent) ? 0 : percent) + '%</a>');
    $('#allCalls a').text(statsObj.totalAll);
    $('#transfCalls a').text(statsObj.totalTransf);
    $('#outCalls a').text(((+statsObj.totalOutAnsw) + (+statsObj.totalOutFailed)).toString());
  }
}

function displayIvrLog(date, from, ivrLog) {
  if(typeof(ivrLog) != 'object' || !ivrLog.length) {
    return;
  }
  $('#ivrLog').remove();
  $('body').append('<div id="ivrLog"><a class="closeIcon right pointer" title="Close"></a>' +
    '<p class="head">' + date + ' ' + from + '</p><div class="logContainer"></div></div>'
  );
  $('#ivrLog .closeIcon').on('click', function() {
    $(this).parent().remove();
  });
  $('#ivrLog .logContainer').append('<table width="100%"></table>');
  var sel = $('#ivrLog .logContainer > table');
  for(var i = 0; i < ivrLog.length; i++) {
    sel.append('<tr><td>' + ivrLog[i].time.substring(11) + '</td><td>' + translate(['ivrActions', ivrLog[i].action]) + ' ' + (ivrLog[i].action == 'ivrLang' ? translate(['LanguagesSelf', ivrLog[i].param]) : ivrLog[i].param) + '</td></tr>');
  }
}
function displayCallNotes(date, from, notes) {
  if(typeof(notes) != 'object' || !Object.keys(notes).length) {
    return;
  }
  $('#ivrLog').remove();
  $('body').append('<div id="ivrLog"><a class="closeIcon right pointer" title="Close"></a>' +
    '<p class="head">' + date + ' ' + from + '</p><div class="logContainer"></div></div>'
  );
  $('#ivrLog .closeIcon').on('click', function() {
    $(this).parent().remove();
  });
  $('#ivrLog .logContainer').append('<table width="100%"></table>');
  var sel = $('#ivrLog .logContainer > table');
  var agentList = Object.keys(notes);
  for(var i = 0; i < agentList.length; ++i) {
    var tModified = new Date(notes[agentList[i]].tModified);
    var agentNotes = '';
    var noteFields = Object.keys(notes[agentList[i]].note);
    for(var j = 0; j < noteFields.length; ++j) {
      if(noteFields[j] == '') continue;
      agentNotes += '<b>' + noteFields[j] + '</b>: ' + notes[agentList[i]].note[noteFields[j]] + '<br/>';
    }
    if(notes[agentList[i]].note['']) agentNotes += notes[agentList[i]].note[''];
    sel.append('<tr><td>' + agentList[i] + '<br/>' + tModified.toTimeString().substring(0,8) + '</td><td>' + agentNotes + '</td></tr>');
  }
}

function getFullObjArr() {
  var objArr = [];
  if(sessionParams.access.monitor.length) {
    if(sessionParams.access.monitor[0] == 'anyOne') {
      var groups = Object.keys(monitorData.group);
      var agents = Object.keys(monitorData.agent);
      objArr = objArr.concat(groups,agents);
    } else {
      for(var i = 0; i < sessionParams.access.monitor.length; i++) {
        if(monitorData.group[sessionParams.access.monitor[i]]) {
          objArr.push(sessionParams.access.monitor[i]);
          objArr = objArr.concat(monitorData.group[sessionParams.access.monitor[i]].agents);
        } else if(monitorData.agent[sessionParams.access.monitor[i]]) {
          objArr.push(sessionParams.access.monitor[i]);
        }
      }
    }
  }
  if(objArr.indexOf(sessionParams.name) < 0 && monitorData.agent[sessionParams.name]) {
    objArr.push(sessionParams.name);
  }
  return objArr;
}

function displayPreFilter() {
  if($('#prefilterSettings').length) {
    $('#prefilterSettings .closeIcon').trigger('click');
    return;
  }
  $('body').append('<div id="prefilterSettings"><p class="head yellow">' + translate('add') + ' ' + translate('filter') + 
    '<a class="closeIcon right pointer" title="Close"></a></p>' +
    '<div class="prefilterContainer">' +
    '<p id="prefilterObjects"></p><div id="objectList"></div>' +
    '<p><input type="text" id="prefilterFilter" data-value="' + translate('prefilterPrompt') + '" class="bg input" />' +
    '<button id="addPrefilterObject" class="ff0 button right">' + translate('add') + '</button></p>' +
    '</div></div>'
  );
  $('#prefilterSettings .closeIcon').on('click', function() {
    $(this).parent().parent().remove();
    $('#str_req').removeClass('pink').addClass('ff0');
  });
  var objArr = getFullObjArr();
  for(var i = 0; i < objArr.length; i++) {
    $('#objectList').append('<p class="pointer" data-value="' + objArr[i] + '">' + objArr[i] + '</p>');
  }
  for(var i = 0; i < prefilterObjects.length; i++) {
    $('#prefilterObjects').append(
      '<span><a class="pointer xIcon" data-value="' + prefilterObjects[i] + '">' + prefilterObjects[i] + '</a></span>'
    );
  }
  $('#prefilterObjects a.xIcon').on('click', function() {
    var objName = $(this).attr('data-value');
    var index = prefilterObjects.indexOf(objName);
    if(index > -1) {
      prefilterObjects.splice(index, 1);
    }
    $(this).parent().remove();
    $('#str_req').removeClass('ff0').addClass('pink');
  });
  $('#objectList p').on('click', function() {
    if($(this).hasClass('active')) return;
    $(this).addClass('active').siblings('.active').removeClass('active');
    $('#prefilterFilter').val($(this).attr('data-value'));
  });
  $('#addPrefilterObject').on('click', function() {
    var objName = $('#prefilterFilter').val();
    if(!objName || $('#prefilterObjects a[data-value="' + objName + '"]').length || objArr.indexOf(objName) < 0) return;
    $('#prefilterObjects').append('<span><a class="pointer xIcon" data-value="' + objName + '">' + objName + '</a></span>');
    prefilterObjects.push(objName);
    $('#str_req').removeClass('ff0').addClass('pink');
    $('#prefilterObjects a.xIcon:last-child').on('click', function() {
      var objName = $(this).attr('data-value');
      var index = prefilterObjects.indexOf(objName);
      if(index > -1) {
        prefilterObjects.splice(index, 1);
      }
      $(this).parent().remove();
      $('#str_req').removeClass('ff0').addClass('pink');
    });
  });
  $('#prefilterFilter').on('focus',function(){
		var t = $(this);
		if(t.val()==t.attr('data-value'))
			t.val('');
	}).on('blur',function(){
		var t = $(this);
		if(t.val()=='')
			t.val(t.attr('data-value'));
	});
  $('#prefilterFilter').trigger('blur');
  $('#prefilterFilter').on('keyup', function() {
    if(this.value.replace(/\s/g).length && this.value != $(this).attr('data-value')) {
      $('#objectList p:contains("' + this.value + '")').show();
      $('#objectList p:not(p:contains("' + this.value + '"))').hide();
    } else {
      $('#objectList p').show();
    }
  });
}
function displayUpStats() {
  if($('#upStats').length) return;
  $('body').append('<div id="upStats" class="pointer"><img src="' + filesRef + 'arrowtop.svg" width="64" height="64" /></div>');
  $('#upStats').on('click', function() {
    window.scrollTo(0, 0);
  });
}
function cancelReport() {
  $('#statsProgress').remove();
  cancelStatisticsReport();
}
function displayStatsProgress(param) {
  $('#statsProgress').remove();
  if(!param) return;
  var val;
  if(isNaN(param)) { // report hyperlink
    var href = location.origin + '/Session/' + sessionId + '/WebFile/' + param;
    val = '<a id="statsProgress" style="border-bottom-right-radius: 0px; border-top-left-radius: 0px; padding: 0px 5px;" class="yellow right pointer" href="' + href + '" target="_blank">' + translate('expex') + '</a>';
  } else { // report progress
    val = '<span id="statsProgress" style="border-bottom-right-radius: 0px; border-top-left-radius: 0px; padding: 0px 5px;" class="yellow right">' + translate('statsProgress') + ' ' + param + '% <a class="right pointer" style="margin-left:5px;color:#788088;" onclick="cancelReport();">' + translate('stop') + '</a></span>';
  }
  $('body > .header').append(val);
}

function secToTimeStr(ts) {
  var h = Math.floor(ts/3600); var m = Math.floor((ts-h*3600)/60); var s = ts - h*3600 - m*60;
  return (h < 10 ? "0"+h : h) + ":" + ("0"+m).substr(-2) + ":" + ("0"+s).substr(-2);
}

//
// ccAgentStatusTimelime
//

var statusClassMap = { on: "timelineStatusOn", off: "timelineStatusOff", acw: "timelineStatusAcw", break: "timelineStatusBreak", future: "timelineStatusFuture" };

function getTimeframeLength() { return timeframe.secondHour - timeframe.firstHour; }
function getSepPerHour() { // returns scale separator count per hour
  var timeDiff = getTimeframeLength();
  if(timeDiff <= 4) return 6;
  else if(timeDiff <= 12) return 2;
  else return 1;
}
function getTimeScaleVals() {
  var result = []; var sepPerHour = getSepPerHour();
  for(var i = 0; i < getSepPerHour()*getTimeframeLength(); ++i)
    result[i] = secToTimeStr(timeframe.firstHour*3600+i*3600/sepPerHour).substr(0,5);
  return result;
}

function Timespan(statusName,statusInfo,length,statusDate,ts) {
  var self = this;
  this.statusName = statusName; this.statusInfo = statusInfo; this.length = length;
  this.span = document.createElement("span");
  this.span.classList.add("timelineTimespan"); this.span.classList.add(statusClassMap[this.statusName]);
  if(this.statusInfo!="-") {
    var color = translate(['agentStatusInfoColor',this.statusName,this.statusInfo]);
    if(color !== this.statusInfo) this.span.style = "background-color:"+color;
  }
  this.span.style.width = this.length*100/(getTimeframeLength()*3600) + "%";
  var statusTime = new Date(statusDate);
  if(this.statusName != 'future') {
    this.span.title = this.statusInfo!="-" ? translate(['agentStatusInfo',this.statusName,this.statusInfo]) : translate(['agentStatus',this.statusName]);
    this.span.title += "\n" + secToTimeStr(ts) + "-" + secToTimeStr(ts+length);
  }
}
Timespan.prototype.putInto = function(container) { container.append(this.span); };
function AgentTimeline(name,statusArr) {
  var self = this;
  this.name = name;
  this.div = document.createElement("div"); this.div.classList.add("timelineTimespanContainer");
  var firstFlag = false;
  var totalLen = 0;
  if(statusArr.length == 1) {
    var timespan = new Timespan(statusArr[0].status,statusArr[0].info,getTimeframeLength()*3600,statusArr[0].date,statusArr[0].ts); timespan.putInto(this.div);
    return;
  }
  for(var i = 0; i < statusArr.length; ++i) {
    if(statusArr[i].ts < timeframe.firstHour*3600) continue;
    if(!firstFlag && i > 0) {
      var firstTimespan = new Timespan(statusArr[i-1].status,statusArr[i-1].info,statusArr[i].ts-timeframe.firstHour*3600,statusArr[i-1].date,timeframe.firstHour*3600); firstTimespan.putInto(this.div);
      firstFlag = true; totalLen += statusArr[i].ts-timeframe.firstHour*3600;
    }
    if(totalLen + statusArr[i].len > getTimeframeLength()*3600) {
      var timespan = new Timespan(statusArr[i].status,statusArr[i].info,statusArr.length == 1 ? getTimeframeLength()*3600 : timeframe.secondHour*3600-statusArr[i].ts,statusArr[i].date,statusArr[i].ts); timespan.putInto(this.div);
      return;
    }
    var timespan = new Timespan(statusArr[i].status,statusArr[i].info,statusArr[i].len,statusArr[i].date,statusArr[i].ts); timespan.putInto(this.div);
    totalLen += statusArr[i].len;
    if(!firstFlag) firstFlag = true;
  }
  if(!firstFlag) {
    var iEnd = statusArr.length - 1;
    var timespan = new Timespan(statusArr[iEnd].status,statusArr[iEnd].info,getTimeframeLength()*3600,statusArr[iEnd].date,statusArr[iEnd].ts); timespan.putInto(this.div);
  }
}
AgentTimeline.prototype.getName = function() { return this.name; };
AgentTimeline.prototype.putInto = function(container) { container.append(this.div); };

function processAgentTimeline(result) {
  statistics.agentStatusTimeline = result;
  var agentArr = Object.keys(result);
  var mainContainer = document.getElementById("mainTimelineContainer"); var timeScaleContainer = document.getElementById("timelineScaleContainer");
  var timeScaleVals = getTimeScaleVals();
  for(var i = 0; i < getSepPerHour()*getTimeframeLength(); ++i) {
    var sepTs = document.createElement("span"); sepTs.classList.add("tableCell"); sepTs.classList.add("timelineScaleLabel"); sepTs.textContent = timeScaleVals[i];
    timeScaleContainer.append(sepTs);
  }
  for(var i = 0; i < agentArr.length; ++i) {
    var agentTimeline = new AgentTimeline(agentArr[i],result[agentArr[i]]);
    var agentContainer = document.createElement("div"); agentContainer.classList.add("tableRow");
    var nameContainer = document.createElement("div"); nameContainer.classList.add("tableCell"); nameContainer.classList.add("labelCell");
    nameContainer.textContent = agentTimeline.getName();
    var timespanContainer = document.createElement("div"); timespanContainer.classList.add("tableCell"); timespanContainer.classList.add("dataCell");
    agentTimeline.putInto(timespanContainer);
    for(var j = 0; j < getSepPerHour()*getTimeframeLength()-1; ++j) { var sep = document.createElement("span"); sep.classList.add("timelineSep"); sep.style.left = (j+1)*100/(getSepPerHour()*getTimeframeLength()) + "%"; timespanContainer.append(sep); }
    agentContainer.append(nameContainer); agentContainer.append(timespanContainer); mainContainer.append(agentContainer);
  }
}
